﻿using DressAtelierContracts.BindingModels;
using DressAtelierContracts.BusinessLogicContracts;
using DressAtelierContracts.SearchModels;
using DressAtelierContracts.StorageContracts;
using DressAtelierContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DressAtelierBusinessLogic.BusinessLogic
{
    public class DressLogic : IDressLogic
    {
        private readonly ILogger _logger;

        private readonly IDressStorage _dressStorage;

        public DressLogic(ILogger<DressLogic> logger, IDressStorage dressStorage)
        {
            _logger = logger;
            _dressStorage = dressStorage;
        }
        public bool Create(DressBindingModel model)
        {
            CheckModel(model);
            if(_dressStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Update(DressBindingModel model)
        {
            CheckModel(model);
            if (_dressStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(DressBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. ID:{ID}", model.ID);
            if (_dressStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public DressViewModel? ReadElement(DressSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. DressName:{DressName}.ID:{ ID}", model.DressName, model.ID);

            var element = _dressStorage.GetElement(model);

            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. ID:{ID}", element.ID);
            return element;
        }

        public List<DressViewModel>? ReadList(DressSearchModel? model)
        {
            _logger.LogInformation("ReadList. DressName:{DressName}. ID:{ ID}", model?.DressName, model?.ID);

            var list = model == null ? _dressStorage.GetFullList() : _dressStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(DressBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.DressName))
            {
                throw new ArgumentNullException("Invalid name of dress",nameof(model.DressName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Price of dress should be higher than 0", nameof(model.Price));
            }
            _logger.LogInformation("Product. DressName:{DressName}. Cost:{Cost}. ID: {ID}", model.DressName, model.Price, model.ID);

            var element = _dressStorage.GetElement(new DressSearchModel
            {
                DressName = model.DressName,
            });

            if (element != null && element.ID != model.ID)
            {
                throw new InvalidOperationException("Product with such name already exists");
            }
        }
    }
}
